{#
  Copyright (C) 2020 CERN.
  Copyright (C) 2020 Northwestern University.
  Copyright (C) 2021 TU Wien.
  Copyright (C) 2022 New York University.

  Invenio RDM Records is free software; you can redistribute it and/or modify
  it under the terms of the MIT License; see LICENSE file for more details.
-#}

<div class="sidebar-container">
  <h2 class="ui medium top attached header mt-0">{{ _('Versions')}}</h2>
  <div id="record-versions" class="ui segment rdm-sidebar bottom attached pl-0 pr-0 pt-0">
    <div class="versions">
      <div id="recordVersions" data-record='{{ record | tojson }}' data-preview='{{ is_preview | tojson }}'>
        <div class="rel-p-1"></div>
        <div class="ui fluid placeholder rel-mr-1 rel-ml-1"></div>
        <div class="header">
          <div class="line"></div>
          <div class="line"></div>
          <div class="line"></div>
        </div>
      </div>
    </div>
  </div>
</div>
